# -*- coding: utf-8 -*-
"""
Created on Thu Mar 11 11:54:12 2021

@author: Rontc
"""


def createSession(session, filepath, existing_data=False):
    # create file paths for the session, raw and synced videos

    pathList = []

    recordPath = filepath / "Data"
    recordPath.mkdir(exist_ok=True)

    session.sessionPath = recordPath / session.sessionID
    # if existing_data:
    # session.sessionPath.mkdir(exist_ok = True)
    # else:
    session.sessionPath.mkdir(exist_ok=True)
    pathList.append(session.sessionPath)

    session.rawVidPath = session.sessionPath / "RawVideos"
    session.rawVidPath.mkdir(exist_ok=True)
    pathList.append(session.rawVidPath)

    session.syncedVidPath = session.sessionPath / "SyncedVideos"
    session.syncedVidPath.mkdir(exist_ok=True)
    pathList.append(session.syncedVidPath)

    session.calVidPath = session.sessionPath / "CalVideos"
    session.calVidPath.mkdir(exist_ok=True)
    pathList.append(session.calVidPath)

    session.openPoseDataPath = session.sessionPath / "OpenPoseData"
    session.openPoseDataPath.mkdir(exist_ok=True)
    pathList.append(session.openPoseDataPath)

    session.mediaPipeDataPath = session.sessionPath / "MediaPipeData"
    session.mediaPipeDataPath.mkdir(exist_ok=True)
    pathList.append(session.mediaPipeDataPath)

    session.dlcDataPath = session.sessionPath / "DLCdata"
    session.dlcDataPath.mkdir(exist_ok=True, parents=True)
    pathList.append(session.dlcDataPath)

    session.imOutPath = session.sessionPath / "imOut"
    session.imOutPath.mkdir(exist_ok=True)
    pathList.append(session.imOutPath)

    session.dataArrayPath = session.sessionPath / "Data Arrays"
    session.dataArrayPath.mkdir(exist_ok=True)
    pathList.append(session.dataArrayPath)

    # create a list of paths that is passed to the recording script
    # pathList = [session.sessionPath,session.rawVidPath,session.syncedVidPath,session.openPoseDataPath,session.dlcDataPath,session.imOutPath, session.dataArrayPath]

    config_settings, config_yaml = create_config_yaml()
    session.config_settings = config_settings

    yaml_name = session.sessionPath / "{}_config.yaml".format(session.sessionID)
    session.yamlPath = yaml_name
    # print(yaml_name)
    write_config_yaml(session, config_settings, config_yaml, pathList, yaml_name)

    return yaml_name


def createSessionTxt(session, paramDict, rotDict):
    # create a text file listing recording parameters
    parameter_text = session.sessionPath / "sessionSettings.txt"
    text = open(parameter_text, "w")
    text.write("Session ID = %s\n" % (session.sessionID))
    text.write("%s = %s\n" % ("Parameters", paramDict))
    text.write("%s = %s\n" % ("Rotations", rotDict))
    text.close()

folder_setup = ['RawVideos','SyncedVideos','CalVideos','DataArrays','DLCData','OpenPoseData','MediaPipeData','imOut']




# def createSession(session,filepath, existing_data = False):
#     #create file paths for the session, raw and synced videos
    
#     pathList = []
   
#     recordPath = filepath/'Data'
#     recordPath.mkdir(exist_ok= True)
    
#     session.sessionPath = recordPath/session.sessionID
#     #if existing_data:
#         #session.sessionPath.mkdir(exist_ok = True)
#     #else:
#     session.sessionPath.mkdir(exist_ok= True)
#     pathList.append(session.sessionPath)
    
#     session.rawVidPath = session.sessionPath/'RawVideos'
#     session.rawVidPath.mkdir(exist_ok = True)
#     pathList.append(session.rawVidPath)

#     session.syncedVidPath = session.sessionPath/'SyncedVideos'   
#     session.syncedVidPath.mkdir(exist_ok = True)
#     pathList.append(session.syncedVidPath)

#     session.calVidPath = session.sessionPath/'CalVideos'
#     session.calVidPath.mkdir(exist_ok = True)
#     pathList.append(session.calVidPath)

#     session.openPoseDataPath = session.sessionPath/'OpenPoseData'
#     session.openPoseDataPath.mkdir(exist_ok = True)
#     pathList.append(session.openPoseDataPath)

#     session.mediaPipeDataPath = session.sessionPath/'MediaPipeData'
#     session.mediaPipeDataPath.mkdir(exist_ok = True)
#     pathList.append(session.mediaPipeDataPath)
    
#     session.dlcDataPath  =  session.sessionPath / 'DLCdata'
#     session.dlcDataPath.mkdir(exist_ok = True, parents = True)
#     pathList.append(session.dlcDataPath)
    
#     session.imOutPath = session.sessionPath / 'imOut'
#     session.imOutPath.mkdir(exist_ok=True)
#     pathList.append(session.imOutPath)

#     session.dataArrayPath = session.sessionPath/'Data Arrays'
#     session.dataArrayPath.mkdir(exist_ok=True)
#     pathList.append(session.dataArrayPath)
    
#     #create a list of paths that is passed to the recording script
#     #pathList = [session.sessionPath,session.rawVidPath,session.syncedVidPath,session.openPoseDataPath,session.dlcDataPath,session.imOutPath, session.dataArrayPath]
   
    
#     config_settings,config_yaml = create_config_yaml()
#     session.config_settings = config_settings

#     yaml_name = session.sessionPath/'{}_config.yaml'.format(session.sessionID)
#     session.yamlPath = yaml_name
#    # print(yaml_name)
#     write_config_yaml(session,config_settings,config_yaml,pathList,yaml_name)
    
#     return yaml_name

# def createSessionTxt(session,paramDict,rotDict):
#     #create a text file listing recording parameters
#     parameter_text = session.sessionPath/'sessionSettings.txt' 
#     text = open(parameter_text, 'w')
#     text.write("Session ID = %s\n" %(session.sessionID))
#     text.write("%s = %s\n" %("Parameters", paramDict))
#     text.write("%s = %s\n" %("Rotations", rotDict))
#     text.close()
    
    
    

# def create_config_yaml():

# def create_config_yaml():

#     yaml_str = """\
#     # Camera Inputs
#     CamInputs:
#         RotationInputs:
#         ParameterDict:

#     # Paths
#     Paths:
#         sessionPath: 
#         rawVidPath:
#         syncedVidPath:
#         calVidPath:
#         openPoseDataPath:
#         mediaPipeDataPath:
#         dlcDataPath:
#         imOutPath:
#         dataArrayPath:
#         \n
#         """
#     config_yaml = YAML()
#     config_settings = config_yaml.load(yaml_str)

#     return config_settings,config_yaml

# def write_config_yaml(session, config_settings, config_yaml, pathList, yaml_name):

# def write_config_yaml(session,config_settings,config_yaml,pathList,yaml_name):
    
#     for count,key in enumerate(config_settings['Paths'].keys()):
#         config_settings['Paths'][key] = str(pathList[count])
    
#     config_settings['CamInputs']['RotationInputs'] = session.rotationInputs
#     config_settings['CamInputs']['ParameterDict'] = session.parameterDictionary
    

#     #yamlPath = sessionPath/'config_yaml'
#     #print(config_settings,yaml_name)
#     with open(yaml_name, 'w') as outfile:
#         config_yaml.dump(config_settings, outfile)


# def load_config_yaml(yaml_path):
#     config_yaml = YAML()
#     with open(yaml_path,'r') as fp:
#         config_settings = config_yaml.load(fp)
#     return config_settings

# def load_session_paths(session,config_settings):
#     configPaths = config_settings['Paths']
#     session.rawVidPath = Path(configPaths['rawVidPath'])
#     session.syncedVidPath = Path(configPaths['syncedVidPath'])
#     session.calVidPath = Path(configPaths['calVidPath'])
#     session.mediaPipeDataPath = Path(configPaths['mediaPipeDataPath'])
#     session.openPoseDataPath = Path(configPaths['openPoseDataPath'])
#     session.dlcDataPath = Path(configPaths['dlcDataPath'])
#     session.imOutPath = Path(configPaths['imOutPath'])
#     session.dataArrayPath = Path(configPaths['dataArrayPath'])



import threading
import cv2
import time
import pickle


class CamRecordingThread(threading.Thread):
    def __init__(
        self, camID, camInput, videoName, rawVidPath, beginTime, parameterDictionary
    ):
        threading.Thread.__init__(self)
        self.camID = camID
        self.camInput = camInput
        self.videoName = videoName
        self.rawVidPath = rawVidPath
        self.beginTime = beginTime
        self.parameterDictionary = parameterDictionary

    def run(self):
        print("Starting " + self.camID)
        self.timeStamps = CamRecording(
            self.camID,
            self.camInput,
            self.videoName,
            self.rawVidPath,
            self.beginTime,
            self.parameterDictionary,
        )

    def getStamps(self):
        return self.timeStamps


# the recording function that each threaded camera object runs
def CamRecording(
    camID, camInput, videoName, rawVidPath, beginTime, parameterDictionary
):
    # the flag is triggered when the user shuts down one webcam to shut down the rest.
    # normally I'd try to avoid global variables, but in this case it's
    # necessary, since each webcam runs as it's own object.
    global flag
    flag = False

    cv2.namedWindow(camID)  # name the preview window for the camera its showing
    cam = cv2.VideoCapture(camInput, cv2.CAP_DSHOW)  # create the video capture object
    # if not cam.isOpened():
    #         raise RuntimeError('No camera found at input '+ str(camID))
    # pulling out all the dictionary paramters
    exposure = parameterDictionary.get("exposure")
    resWidth = parameterDictionary.get("resWidth")
    resHeight = parameterDictionary.get("resHeight")
    framerate = parameterDictionary.get("framerate")
    codec = parameterDictionary.get("codec")

    cam.set(cv2.CAP_PROP_FRAME_WIDTH, resWidth)
    cam.set(cv2.CAP_PROP_FRAME_HEIGHT, resHeight)
    cam.set(cv2.CAP_PROP_EXPOSURE, exposure)
    fourcc = cv2.VideoWriter_fourcc(*codec)
    # rawPath = filepath/'RawVideos' #creating a RawVideos folder
    # rawPath.mkdir(parents = True, exist_ok = True)
    width = cam.get(cv2.CAP_PROP_FRAME_WIDTH)
    height = cam.get(cv2.CAP_PROP_FRAME_HEIGHT)
    print("width:", width, "height:", height)
    saveRawVidPath = str(
        rawVidPath / videoName
    )  # create a save path for each video to the RawVideos folders
    out = cv2.VideoWriter(saveRawVidPath, fourcc, framerate, (resWidth, resHeight))
    timeStamps = []  # holds the timestamps

    if cam.isOpened():
        success, frame = cam.read()
    else:
        success = False

    while (
        success
    ):  # while the camera is opened, record the data until the escape button is hit
        if flag:  # when the flag is triggered, stop recording and dump the data
            with open(camID, "wb") as f:
                pickle.dump(timeStamps, f)
            break
        success, frame = cam.read()

        cv2.imshow(camID, frame)
        frame_sized = cv2.resize(frame, (resWidth, resHeight))
        # frame_sized = frame
        out.write(frame_sized)
        timeStamps.append(time.time() - beginTime)  # add each timestamp to the list

        key = cv2.waitKey(20)
        if key == 27:  # exit on ESC
            flag = True  # set flag to true to shut down all other webcams
            with open(camID, "wb") as f:
                pickle.dump(timeStamps, f)  # dump the data
            break
    cv2.destroyWindow(camID)
    return timeStamps


# this is how we sync our time frames, based on our recorded timestamps



from freemocap import fmc_anipose
import numpy as np

# from numba import jit

# @jit(nopython=True)
def reconstruct3D(session, data_nCams_nFrames_nImgPts_XYC, confidenceThreshold=0.3):

    if (
        session.cgroup is None
    ):  # load the calibration settings into the session class if it hasn't been already
        calibrationFile = "{}_calibration.yaml".format(session.sessionID)
        session.cameraConfigFilePath = session.sessionPath / calibrationFile
        session.cgroup = fmc_anipose.CameraGroup.load(session.cameraConfigFilePath)

    nCams, nFrames, nImgPts, nDims = data_nCams_nFrames_nImgPts_XYC.shape

    if nDims == 3:

        data_nCams_nFrames_nImgPts_X = np.squeeze(
            data_nCams_nFrames_nImgPts_XYC[:, :, :, 0].copy()
        )
        data_nCams_nFrames_nImgPts_Y = np.squeeze(
            data_nCams_nFrames_nImgPts_XYC[:, :, :, 1].copy()
        )
        confidence = np.squeeze(data_nCams_nFrames_nImgPts_XYC[:, :, :, 2].copy())

        data_nCams_nFrames_nImgPts_X[
            confidence < confidenceThreshold
        ] = np.nan  # replace low confidence points with 'nan'
        data_nCams_nFrames_nImgPts_Y[confidence < confidenceThreshold] = np.nan
        data_nCams_nFrames_nImgPts_XY = np.stack(
            (data_nCams_nFrames_nImgPts_X, data_nCams_nFrames_nImgPts_Y), axis=3
        )

    if nDims == 2:
        data_nCams_nFrames_nImgPts_XY = data_nCams_nFrames_nImgPts_XYC

    dataFlat_nCams_nTotalPoints_XY = data_nCams_nFrames_nImgPts_XY.reshape(
        nCams, -1, 2
    )  # reshape data to collapse across 'frames' so it becomes [numCams, numFrames*numPoints, XY]

    data3d_flat = session.cgroup.triangulate(
        dataFlat_nCams_nTotalPoints_XY, progress=True
    )
    dataReprojerr_flat = session.cgroup.reprojection_error(
        data3d_flat, dataFlat_nCams_nTotalPoints_XY, mean=True
    )

    ##return:
    data_fr_mar_dim = data3d_flat.reshape(nFrames, nImgPts, 3)
    dataReprojErr_fr_mar_err = dataReprojerr_flat.reshape(nFrames, nImgPts)

    return data_fr_mar_dim



import numpy as np
import pandas as pd
import sys
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
import tkinter as tk
from tkinter import Tk, Label, Button, Frame

def TimeSync(session,timeStampData,numCamRange,camNames):    
       
    def CloseNeighb(camera,point): 
            closestPoint = (np.abs(camera - point)).argmin() 
            return closestPoint

def TimeSync(timeStampData, numCamRange, camNames):
    def CloseNeighb(camera, point):
        closestPoint = (np.abs(camera - point)).argmin()
        return closestPoint

    # this section auto-finds the start and end points for our master timeline to the nearest second
    masterTimelineBegin = np.ceil(max(timeStampData.iloc[0]))
    lastPoints = []
    for name in camNames:
        cameraLastPoint = timeStampData[name][
            timeStampData[name].last_valid_index()
        ]  # find the last non-nan value in each camera
        lastPoints.append(cameraLastPoint)
    
    masterTimelineEnd = np.floor(min(lastPoints))  #find where the fastest camera ended, round down, use that as the end point


    #print('end',endList)
    print('intervals:',masterTimelineBegin,masterTimelineEnd)   
    
    preSyncTotalNumFrames = []
    totalFrameIntvl= [] #list for storing frame rates for each camera
    
      
    n = 0; #counter for going through camera names
    for x in numCamRange:
      currentCam = timeStampData[camNames[x]]
      #finds the closest point in each camera to the start and end points of our timeline
      currentCamStart = CloseNeighb(currentCam,masterTimelineBegin) 
      currentCamEnd = CloseNeighb(currentCam,masterTimelineEnd)
    

      print("using frames:", currentCamStart,"-",currentCamEnd,"for",camNames[n])
      n +=1
      currentCamTimeline = currentCam[currentCamStart:currentCamEnd] #grab the times from start to finish for each camera

      currentCamNumFrames = len(currentCamTimeline)

      currentCamFrameInterval = np.mean(np.diff(currentCamTimeline)) #calculate the interval between each frame and take the mean 
      
      totalFrameIntvl.append(currentCamFrameInterval) #add interval to list 
      preSyncTotalNumFrames.append(currentCamNumFrames)
      #print(camNames[x],currentCamStart,currentCamEnd)

    #print(preSyncTotalNumFrames, 'presync')
    #assert preSyncTotalNumFrames.count(preSyncTotalNumFrames[0]) == len(preSyncTotalNumFrames), "Number of frames sliced for each camera pre-syncing is not the same"

    totalAverageIntvl = np.mean(totalFrameIntvl) #find the total average interval across all cameras
    masterTimeline = np.arange(masterTimelineBegin,masterTimelineEnd,totalAverageIntvl) #build a master timeline with the average interval
       
    #now we start the syncing process
    
    frameList = masterTimeline #start a list of frames, with the first row being our master timeline
    timeList = masterTimeline #start a list of timestamps,with the first row being our master timeline
     
    delNum= []; #stores number of deleted frames/camera
    bufNum= []; #stores number of buffered frames/camera
    bufPercentList = []; #stores percentage of deleted frames/camera
    delPercentList = []; #stores percentage of buffered frames/camera
    
    count = 0 #Keeps track of what frame we're on (I think?)
    n = 0; #I don't remember why I did this but I'm sure I'll figure it out later
    

    postSyncTotalNumFrames = []
    for y in numCamRange:
        thisCam = timeStampData[camNames[y]]
        camTimes = []
        # stored times
        camFrames = []
        # stored frames

        count += 1
        # --------------Adjust each camera to start at the first point in the master timeline
        beginFrame = CloseNeighb(thisCam, masterTimeline[0])
        timeDif = masterTimeline[0] - thisCam[beginFrame]
        thisCam = thisCam + timeDif
        
        for z in masterTimeline: #for each point in the master timeline
            closestFrame = CloseNeighb(thisCam, z) #find the closest frame in this camera to each point of the master timeline
            camFrames.append(closestFrame) #add that frame to the list
            camTimes.append(thisCam[closestFrame]) #find the time corresponding to this frame
            
        print("starting detection:",camNames[n]) #now to start finding deleted/buffered slides
        frameList = np.column_stack((frameList,camFrames)) #update our framelist with our new frames
        timeList = np.column_stack((timeList,camTimes)) #update our timelist 
        
    
        #start counters for the number of buffered and deleted slides
        bufCount = 0;
        delCount = 0;

        thisCamNumFrames = len(camFrames)
        postSyncTotalNumFrames.append(thisCamNumFrames)

        for i in range(0,len(camFrames)-1): 
            distanceBetweenFrames = camFrames[i+1] - camFrames[i] #find the distance between adjacent frames
            if distanceBetweenFrames == 1: #these frames are consecutive, do nothing
                None 
            elif distanceBetweenFrames == 0: #we have a buffered slide
                bufCount += 1
                # this section looks at our two timepoints, and finds the which point on the master timeline is closest
                frame1 = abs(masterTimeline[i] - camTimes[i])
                frame2 = abs(masterTimeline[i + 1] - camTimes[i])
                # finds which frame is closest, and sets the other frame as a buffer (indicated by the -1)
                if frame1 > frame2:
                    frameList[i, count] = -1
                    timeList[i, count] = -1
                else:
                    frameList[i + 1, count] = -1
                    timeList[i + 1, count] = -1
            elif distanceBetweenFrames > 1:  # deletion
                delCount += 1
            else:

                print("something else happened")
        #update and calculate our percentages and numbers for buffers/deletions
        delNum.append(round(delCount,1))
        bufNum.append(round(bufCount,1))
        #print(bufCount,len(frameList))
        bufPercent = (bufCount/len(frameList))*100
        delPercent = (delCount/len(frameList))*100
        bufPercentList.append(round(bufPercent,2))
        delPercentList.append(round(delPercent,2))
       
        #print(delNum,bufNum,bufPercentList,delPercentList)
        #print("deleted frames:",delCount)
        #print("buffered frames:",bufCount)
        n +=1

    #print(postSyncTotalNumFrames, 'post-sync')
    assert postSyncTotalNumFrames.count(postSyncTotalNumFrames[0]) == len(postSyncTotalNumFrames), "Number of frames sliced for each camera post-syncing is not the same"

    session.postSyncNumFrames = postSyncTotalNumFrames[0]

    #create our data frame for both times and frames    
    frameTable = pd.DataFrame(frameList)   
    columnNames = ['Master Timeline'] + camNames
    frameTable.columns = columnNames
    timeTable = pd.DataFrame(timeList)
    timeTable.columns = columnNames
    totalFrameRate = [round(1 / intvl, 1) for intvl in totalFrameIntvl]
    frameRate = 1 / totalAverageIntvl  # calculates our framerate
    results = {
        "Cam": camNames,
        "#Del": delNum,
        "%Del": delPercentList,
        "#Buf": bufNum,
        "%Buf": bufPercentList,
        "FPS": totalFrameRate,
    }
    resultTable = pd.DataFrame(
        results, columns=["Cam", "#Del", "%Del", "#Buf", "%Buf", "FPS"]
    )

    # ============================================== plot data
    differenceFrame = timeStampData.diff(axis=0)
    fpsFrame = differenceFrame.pow(-1)
    fig = Figure(figsize=[10, 10])
    fig.patch.set_facecolor("#F0F0F0")
    a = fig.add_subplot(221)
    a.set_xlabel("Frame")
    a.set_ylabel("Time(s)")
    timeStampData.plot(ax=a, title="Camera timestamps")
    b = fig.add_subplot(222)
    b.set_xlabel("Frame")
    b.set_ylabel("Frame duration (s)")
    differenceFrame.plot(
        ax=b, marker=".", linestyle="none", title="Camera frame duration"
    )
    c = fig.add_subplot(223)
    c.set_xlabel("Time (s)")
    differenceFrame.plot.hist(
        ax=c, bins=6, alpha=0.5, xlim=[0, 0.1], title="Frame interval distribution (s)"
    )
    d = fig.add_subplot(224)
    d.set_xlabel("Frames per second")
    fpsFrame.plot.hist(
        ax=d, bins=6, alpha=0.5, xlim=[0, 40], title="Frames per second distribution"
    )
    return frameTable, timeTable, frameRate, resultTable, fig


# function to trim our videos


class proceedGUI:
    def __init__(self, master, results, figure):
        self.master = master
        self.results = results
        self.figure = figure
        self.proceed = ""
        master.title("Choose File Path")

        bottom_frame = Frame(self.master, height=1000)
        bottom_frame.pack(side=tk.LEFT)
        self.label = Label(bottom_frame, text=results.to_string(index=False))
        self.label.pack(side=tk.TOP)
        self.go_button = Button(bottom_frame, text="Proceed", command=self.destroy)
        self.stop_button = Button(bottom_frame, text="Quit", command=self.stop)

        self.canvas = FigureCanvasTkAgg(self.figure, master=self.master)
        self.canvas.draw()
        self.canvas.get_tk_widget().pack(side=tk.LEFT, fill="both", expand=1)

        self.stop_button.pack(side=tk.RIGHT)
        self.go_button.pack(side=tk.RIGHT)

    def stop(self):
        self.master.destroy()
        sys.exit("Quitting Program")

    def destroy(self):
        self.proceed = True
        self.master.destroy()



from pathlib import Path


import numpy as np
from rich.progress import Progress
from ruamel.yaml import YAML
import cv2
import mediapipe as mp

# from numba import jit

mp_drawing = mp.solutions.drawing_utils
mp_holistic = mp.solutions.holistic


def runMediaPipe(session, dummyRun=False):

    mediaPipe_jsonPathList = []  # list to hold the paths to the json files
    mediaPipe_imgPathList = []
    mediaPipe_imgPathList_yaml = []
    mediaPipe_jsonPathList_yaml = []

    with mp_holistic.Holistic() as holistic:
        eachCamerasData = []  # Create an empty list that holds each cameras data
        for (
            thisVidPath
        ) in (
            session.syncedVidPath.iterdir()
        ):  # Run MediaPipe 'Holistic' (body, hands, face) tracker on each video in the raw video folder
            if (
                thisVidPath.suffix == ".mp4"
            ):  # NOTE - at some point we should build some list of 'synced video names' and check against that

                vidPath = session.mediaPipeDataPath / thisVidPath.stem
                jsonPath = vidPath / "json"
                jsonPath.mkdir(
                    parents=True, exist_ok=True
                )  # this camera's json files (with keypoints)
                imgPath = vidPath / "images"
                imgPath.mkdir(parents=True, exist_ok=True)
                mediaPipe_jsonPathList.append(jsonPath)
                mediaPipe_imgPathList.append(imgPath)
                mediaPipe_imgPathList_yaml.append(str(imgPath))
                mediaPipe_jsonPathList_yaml.append(str(jsonPath))
                mediaPipe_dataList = []  # Create an empty list for mediapipes data
                if not dummyRun:
                    cap = cv2.VideoCapture(str(thisVidPath))

                    frameNum = -1
                    numFrames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
                    success, image = cap.read()  # load first image from video

                    with Progress() as progress:
                        progressBar = progress.add_task(
                            "[magenta]MediaPiping: {}".format(thisVidPath.name),
                            total=numFrames,
                        )  # make progressbar

                        while success:

                            if frameNum % 5 == 0:
                                progress.update(
                                    progressBar, advance=5
                                )  # increment progress bar everyh 5th frame

                            frameNum += 1

                            image_height, image_width, _ = image.shape

                            results = holistic.process(
                                cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
                            )  # THIS IS WHERE THE MAGIC HAPENS (ps. Convert the BGR image to RGB before processing.)

                            try:
                                results.pose_landmarks.landmark
                                try:
                                    results.left_hand_landmarks.landmark
                                    try:
                                        results.right_hand_landmarks.landmark
                                        try:
                                            results.face_landmarks.landmark
                                            f = 9
                                        except:
                                            pass
                                    except:
                                        pass
                                except:
                                    pass
                            except:
                                pass

                            mediaPipe_dataList.append(
                                results
                            )  # Append data to mediapipe data list

                            saveImages = True
                            if saveImages:
                                # Draw pose, left and right hands, and face landmarks on the image.
                                annotated_image = image.copy()
                                mp_drawing.draw_landmarks(
                                    annotated_image,
                                    results.face_landmarks,
                                    mp_holistic.FACE_CONNECTIONS,
                                )
                                mp_drawing.draw_landmarks(
                                    annotated_image,
                                    results.left_hand_landmarks,
                                    mp_holistic.HAND_CONNECTIONS,
                                )
                                mp_drawing.draw_landmarks(
                                    annotated_image,
                                    results.right_hand_landmarks,
                                    mp_holistic.HAND_CONNECTIONS,
                                )
                                mp_drawing.draw_landmarks(
                                    annotated_image,
                                    results.pose_landmarks,
                                    mp_holistic.POSE_CONNECTIONS,
                                )

                                # save annotated image
                                frameName = str(frameNum).zfill(6)

                                cv2.imwrite(
                                    str(imgPath) + "/" + frameName + ".png",
                                    annotated_image,
                                )

                            # load the next image (will break `while-loop` on the last frame)
                            success, image = cap.read()  # load next image from video
                            f = 9
                eachCamerasData.append(
                    mediaPipe_dataList
                )  # Append that cameras data for every frame to the camera datalist

    session.image_height = image_height
    session.image_width = image_width

    session.mediaPipe_jsonPathList = mediaPipe_jsonPathList
    session.mediaPipe_imgPathList = mediaPipe_imgPathList

    yaml = YAML()
    data = yaml.load(session.yamlPath)
    data["mediaPipe_imgPathList"] = mediaPipe_imgPathList_yaml
    data["mediaPipe_jsonPathList"] = mediaPipe_jsonPathList_yaml
    yaml.dump(data, session.yamlPath)
    session.mediaPipeData = eachCamerasData


# @jit(nopython=True)
def parseMediaPipe(session):
    numCams = len(session.mediaPipeData)  # Get number of cameras
    numFrames = len(session.mediaPipeData[0])  # Get number of frames
    # numBodyPoints = len(np.max(session.mediaPipeData[0][:].pose_landmarks.landmark[:]))#Get number of body points
    # numFacePoints = len(np.max(session.mediaPipeData[0][:].face_landmarks.landmark[:]))#Get number of face points
    # numLeftHandPoints = len(np.max(session.mediaPipeData[0][:].left_hand_landmarks.landmark[:]))#Get number of right hand points
    # numRightHandPoints = len(np.max(session.mediaPipeData[0][:].right_hand_landmarks.landmark[:]))#Get number of left hand points
    numBodyPoints = 33
    numFacePoints = 468
    numHandPoints = 21

    numTrackedPoints = (
        numBodyPoints + numHandPoints * 2 + numFacePoints
    )  # Get total points

    # Create  array of nans the size of number of cams, frame, points, XYC
    mediaPipeData_nCams_nFrames_nImgPts_XYC = np.empty(
        (int(numCams), int(numFrames), int(numTrackedPoints), 3)
    )  # create empty array
    mediaPipeData_nCams_nFrames_nImgPts_XYC[:] = np.NaN  # Fill it with NaNs!

    for camNum in range(numCams):  # Loop through each camera
        for frNum in range(numFrames):  # Loop through each frame
            # make empty arrays for thisFrame's data
            thisFrame_X = np.empty(numTrackedPoints)
            thisFrame_X[:] = np.nan
            thisFrame_Y = thisFrame_X.copy()
            thisFrame_C = thisFrame_X.copy()

            fullFrame = True
            if frNum == 107:
                f = 9
            try:
                # pull out ThisFrame's mediapipe data (`mpData.pose_landmarks.landmark` returns something iterable ¯\_(ツ)_/¯)
                thisFrame_poseDataLandMarks = session.mediaPipeData[camNum][
                    frNum
                ].pose_landmarks.landmark  # body ('pose') data
                # stuff body data into pre-allocated nan array
                thisFrame_X[:numBodyPoints] = [
                    pp.x for pp in thisFrame_poseDataLandMarks
                ]  # PoseX data - Normalized screen coords (in range [0, 1]) - need multiply by image resultion for pixels
                thisFrame_Y[:numBodyPoints] = [
                    pp.y for pp in thisFrame_poseDataLandMarks
                ]  # PoseY data
                thisFrame_C[:numBodyPoints] = [
                    pp.visibility for pp in thisFrame_poseDataLandMarks
                ]  #'visibility' is MediaPose's 'confidence' measure in range [0,1]
            except:
                fullFrame = False

            # Right hand data
            try:
                thisFrame_rHandDataLandMarks = session.mediaPipeData[camNum][
                    frNum
                ].right_hand_landmarks.landmark  # right hand data
                thisFrame_X[numBodyPoints + 1 : numBodyPoints + 1 + numHandPoints] = [
                    pp.x for pp in thisFrame_rHandDataLandMarks
                ]  # PoseX data - Normalized screen coords (in range [0, 1]) - need multiply by image resultion for pixels
                thisFrame_Y[numBodyPoints + 1 : numBodyPoints + 1 + numHandPoints] = [
                    pp.y for pp in thisFrame_rHandDataLandMarks
                ]  # PoseY data
                thisFrame_C[numBodyPoints + 1 : numBodyPoints + 1 + numHandPoints] = [
                    pp.visibility for pp in thisFrame_rHandDataLandMarks
                ]  #'visibility' is MediaPose's 'confidence' measure in range [0,1]
            except:
                fullFrame = False

            # Left hand data
            try:
                thisFrame_lHandDataLandMarks = session.mediaPipeData[camNum][
                    frNum
                ].left_hand_landmarks.landmark  # left hand data
                thisFrame_X[
                    numBodyPoints
                    + numHandPoints
                    + 1 : numBodyPoints
                    + 1
                    + numHandPoints * 2
                ] = [
                    pp.x for pp in thisFrame_lHandDataLandMarks
                ]  # PoseX data - Normalized screen coords (in range [0, 1]) - need multiply by image resultion for pixels
                thisFrame_Y[
                    numBodyPoints
                    + numHandPoints
                    + 1 : numBodyPoints
                    + 1
                    + numHandPoints * 2
                ] = [
                    pp.y for pp in thisFrame_lHandDataLandMarks
                ]  # PoseY data
                thisFrame_C[
                    numBodyPoints
                    + numHandPoints
                    + 1 : numBodyPoints
                    + 1
                    + numHandPoints * 2
                ] = [
                    pp.visibility for pp in thisFrame_lHandDataLandMarks
                ]  #'visibility' is MediaPose's 'confidence' measure in range [0,1]
            except:
                fullFrame = False

            # FaceMeshData
            try:
                thisFrame_faceDataLandMarks = session.mediaPipeData[camNum][
                    frNum
                ].face_landmarks.landmark  # face (mesh) data
                thisFrame_X[numBodyPoints + numHandPoints * 2 + 1 : -1] = [
                    pp.x for pp in thisFrame_faceDataLandMarks
                ]  # PoseX data - Normalized screen coords (in range [0, 1]) - need multiply by image resultion for pixels
                thisFrame_Y[numBodyPoints + numHandPoints * 2 : -1] = [
                    pp.y for pp in thisFrame_faceDataLandMarks
                ]  # PoseY data
                # NOTE - There's also Z data in here
                thisFrame_C[numBodyPoints + numHandPoints * 2 : -1] = [
                    pp.visibility for pp in thisFrame_faceDataLandMarks
                ]  #'visibility' is MediaPose's 'confidence' measure in range [0,1]
            except:
                fullFrame = False

            if fullFrame:
                f = 9

            # stuff this frame's data into pre-allocated mediaPipeData_.... array
            mediaPipeData_nCams_nFrames_nImgPts_XYC[camNum, frNum, :, 0] = thisFrame_X
            mediaPipeData_nCams_nFrames_nImgPts_XYC[camNum, frNum, :, 1] = thisFrame_Y
            mediaPipeData_nCams_nFrames_nImgPts_XYC[camNum, frNum, :, 2] = thisFrame_C

    # convert from normalized screen coordinates to pixel coordinates
    mediaPipeData_nCams_nFrames_nImgPts_XYC[:, :, :, 0] *= session.image_height
    mediaPipeData_nCams_nFrames_nImgPts_XYC[:, :, :, 1] *= session.image_width

    np.save(
        session.dataArrayPath / "mediaPipe_2d.npy",
        mediaPipeData_nCams_nFrames_nImgPts_XYC,
    )

    return mediaPipeData_nCams_nFrames_nImgPts_XYC



from pathlib import Path
from ruamel.yaml import YAML
import datetime
import sys
import pandas as pd
import numpy as np

from freemocap.webcam import recordGUI, camsetup
from freemocap import recordingconfig, fmc_anipose


def initialize(session, stage, board):

    print("Starting initialization for stage {}".format(stage))
    # if stage == 1:
    filepath = Path.cwd()

    session.board = board

    # %% Stage One Initialization
    if stage == 1:

        # create session ID
        sessionID_in = datetime.datetime.now().strftime("sesh_%y-%m-%d_%H%M%S")
        session.sessionID = sessionID_in

    #load the parameters YAML and extract the saved parameters if possible, and the default otherwise       
        here = Path(__file__).parent
        parameter_path = here/'parameters.yaml'
        parameters_yaml = YAML()
        parameters = parameters_yaml.load(parameter_path)

        try:
            rotation_entry = parameters["saved"]["rotations"]
            parameter_entry = parameters["saved"]["parameters"]
        except:
            print("Could not load saved parameters, using default parameters")
            rotation_entry = parameters['default']['rotations']
            parameter_entry = parameters['default']['parameters']      

        proceedToRecording = False #create this boolean, set it to false, and if the user wants to record
                                   #later in the pipeline, it will be set to true

    #run the GUI to get the tasks, the cams chosen, the camera settings, and the session ID
        task, cam_inputs, rotDict, paramDict, session.sessionID = recordGUI.RunGUI(sessionID_in,rotation_entry,parameter_entry)
    
    #update the saved parameters in the YAML
        #recordingconfig.rotation_settings['saved'] = rotDict
        #recordingconfig.camera_parameters['saved'] = paramDict
        parameters['saved']['rotations'] = rotDict
        parameters['saved']['parameters'] = paramDict
        parameters_yaml.dump(parameters, parameter_path)

    #save recording parameters to the config yaml

        
    #create a list from the rotation dictionary to be used in running webcams
        rotation_input = list(rotDict.values())

        if task == "setup":
            # run setup processes, and then check if th user wants to proceed to recording
            camsetup.RunSetup(cam_inputs, rotation_input, paramDict)
            proceedToRecording, session.sessionID = recordGUI.RunProceedtoRecordGUI(
                sessionID_in
            )

        elif task == "record":
            proceedToRecording = True

        if proceedToRecording:
            # create these session properties to be used later in the pipeline
            session.cam_inputs = cam_inputs
            session.parameterDictionary = paramDict
            session.rotationInputs = rotation_input

            #create a config yaml and text file for this session
            session.start_session(session.parameterDictionary,session.rotationInputs)
            session.session_settings['recording_parameters']['RotationInputs'] = rotDict
            session.session_settings['recording_parameters']['ParameterDict'] = paramDict
            #configyaml = YAML()
            #sessionsettings = configyaml.load(session.session_yaml_path)
            #sessionsettings['recording_parameters']['RotationInputs'] = rotDict
            #sessionsettings['recording_parameters']['ParameterDict'] = paramDict
            #configyaml.dump(sessionsettings, session.session_yaml_path)
            #print(sessionsettings)
            #config_yaml_path = recordingconfig.createSession(session,filepath)
            #recordingconfig.createSessionTxt(session,paramDict,rotDict)
            
            print('Proceeding to Stage One')        
    
        else:
        
            sys.exit('Recording Cancelled')
# %% Stage Two Initalization
    if stage == 2:
        # session.sessionPath = filepath/'Data'/session.sessionID #create a session path based on the sessionID

        # #load the config yaml for this session, and add all the paths to the session file
        # session.yamlPath = session.sessionPath/'{}_config.yaml'.format(session.sessionID)
        # session.config_settings = recordingconfig.load_config_yaml(session.yamlPath) #config settings = paths and camera parameter inputs
        # recordingconfig.load_session_paths(session,session.config_settings) #add paths to session class
        session.initialize(stage)

        #from the config settings add the camera input parameters to parameter dictionary
        session.parameterDictionary = session.session_settings['recording_parameters']['ParameterDict']
        session.rotationInputs = session.session_settings['recording_parameters']['RotationInputs']

            print("Proceeding to Stage One")

        else:

            sys.exit("Recording Cancelled")
    # %% Stage Two Initalization
    if stage == 2:
        session.sessionPath = (
            filepath / "Data" / session.sessionID
        )  # create a session path based on the sessionID

        # load the config yaml for this session, and add all the paths to the session file
        session.yamlPath = session.sessionPath / "{}_config.yaml".format(
            session.sessionID
        )
        session.config_settings = recordingconfig.load_config_yaml(
            session.yamlPath
        )  # config settings = paths and camera parameter inputs
        recordingconfig.load_session_paths(
            session, session.config_settings
        )  # add paths to session class

        # from the config settings add the camera input parameters to parameter dictionary
        session.parameterDictionary = session.config_settings["CamInputs"][
            "ParameterDict"
        ]
        session.rotationInputs = session.config_settings["CamInputs"]["RotationInputs"]

        # initialize the path to the timestamp csv
        csvName = session.sessionID + ".csv"
        csvPath = session.sessionPath / csvName

        # read CSV data, turn it into a data frame
        timeStampData = pd.read_csv(csvPath)
        timeStampData = timeStampData.iloc[:, 1:]

        # get the camIDs and number of cameras (numCamRange) from the dataframe
        camIDs = list(timeStampData.columns)
        numCams = len(camIDs)
        numCamRange = range(len(camIDs))

        # create names for each of the raw videos
        vidNames = []
        for x in numCamRange:
            singleVidName = "raw_cam{}.mp4".format(x + 1)
            vidNames.append(singleVidName)

        # initialize all the session variables we'll need to run the rest of the pipeline
        session.timeStampData = timeStampData
        session.camIDs = camIDs
        session.numCamRange = numCamRange
        session.vidNames = vidNames
        session.numCams = numCams

    if stage == 3:

        session.sessionPath = (
            filepath / "Data" / session.sessionID
        )  # create a session path based on the sessionID

        # load the config yaml for this session, and add all the paths to the session file
        session.yamlPath = session.sessionPath / "{}_config.yaml".format(
            session.sessionID
        )

        if (
            session.yamlPath.is_file()
        ):  # if the config yaml exists (from a webcam recording)
            session.config_settings = recordingconfig.load_config_yaml(
                session.yamlPath
            )  # config settings = paths and camera parameter inputs
            recordingconfig.load_session_paths(
                session, session.config_settings
            )  # add paths to session class
        else:  # if it doesn't exist (because of a GoPro/external camera recording)
            recordingconfig.createSession(session, filepath)

    if stage == 4:

        session.sessionPath = filepath / "Data" / session.sessionID

        session.yamlPath = session.sessionPath / "{}_config.yaml".format(
            session.sessionID
        )  # path to the configuration yaml
        session.config_settings = recordingconfig.load_config_yaml(
            session.yamlPath
        )  # config settings = paths and camera parameter inputs
        recordingconfig.load_session_paths(
            session, session.config_settings
        )  # add paths to session class

        for count, thisVidPath in enumerate(
            session.syncedVidPath.glob("*.mp4"), start=1
        ):
            session.numCams = count

    if stage == 6:

        session.sessionPath = filepath / "Data" / session.sessionID

        session.yamlPath = session.sessionPath / "{}_config.yaml".format(
            session.sessionID
        )  # path to the configuration yaml
        session.config_settings = recordingconfig.load_config_yaml(
            session.yamlPath
        )  # config settings = paths and camera parameter inputs
        recordingconfig.load_session_paths(
            session, session.config_settings
        )  # add paths to session class

        if session.useOpenPose:
            session.openPose_imgPathList = session.config_settings[
                "openPose_imgPathList"
            ]

        if session.useMediaPipe:
            session.mediaPipe_imgPathList = session.config_settings[
                "mediaPipe_imgPathList"
            ]

    # if stage == 5:

    #     session.sessionPath = filepath/'Data'/session.sessionID

    #     session.yamlPath = session.sessionPath/'{}_config.yaml'.format(session.sessionID) #path to the configuration yaml
    #     session.config_settings = recordingconfig.load_config_yaml(session.yamlPath) #config settings = paths and camera parameter inputs
    #     recordingconfig.load_session_paths(session,session.config_settings) #add paths to session class

    #     for count,thisVidPath in enumerate(session.syncedVidPath.glob('*.mp4'),start=1):
    #         session.numCams = count

    #     session.mediaPipe_jsonPathList = session.config_settings['mediaPipe_jsonPathList']
    #     session.numCams = len(session.mediaPipe_jsonPathList)

    # if stage == 6:

    #     session.sessionPath = filepath/'Data'/session.sessionID

    #     session.yamlPath = session.sessionPath/'{}_config.yaml'.format(session.sessionID) #path to the configuration yaml
    #     session.config_settings = recordingconfig.load_config_yaml(session.yamlPath) #config settings = paths and camera parameter inputs
    #     recordingconfig.load_session_paths(session,session.config_settings) #add paths to session class

    #     session.mediaPipeData_nCams_nFrames_nImgPts_XY = np.load(session.dataArrayPath/'mediaPipeData_nCams_nFrames_nImgPts_XY.npy')

    if stage == 7:

        session.sessionPath = filepath / "Data" / session.sessionID

        session.yamlPath = session.sessionPath / "{}_config.yaml".format(
            session.sessionID
        )  # path to the configuration yaml
        session.config_settings = recordingconfig.load_config_yaml(
            session.yamlPath
        )  # config settings = paths and camera parameter inputs
        recordingconfig.load_session_paths(
            session, session.config_settings
        )  # add paths to session class

        session.mediaPipe_imgPathList = session.config_settings["mediaPipe_imgPathList"]
        session.openPose_imgPathList = session.config_settings["openPose_imgPathList"]

        # session.openPose_imgPathList = session.config_settings['openPose_imgPathList']
        # session.numCams = len(session.openPose_imgPathList)

    if stage == 8:

        session.sessionPath = filepath / "Data" / session.sessionID

        session.yamlPath = session.sessionPath / "{}_config.yaml".format(
            session.sessionID
        )  # path to the configuration yaml
        session.config_settings = recordingconfig.load_config_yaml(
            session.yamlPath
        )  # config settings = paths and camera parameter inputs
        recordingconfig.load_session_paths(
            session, session.config_settings
        )  # add paths to session class


# %%



import threading
import cv2
import imutils


class VideoSetup(threading.Thread):
    def __init__(self, camID, parameterDictionary, rotNum):
        self.camID = camID
        self.parameterDictionary = parameterDictionary
        self.rotNum = rotNum
        threading.Thread.__init__(self)

    def run(self):
        # print("Starting " + self.previewName)
        self.record(self.parameterDictionary, self.rotNum)

    def record(self, parameterDictionary, rotNum):
        exposure = parameterDictionary.get("exposure")
        resWidth = parameterDictionary.get("resWidth")
        resHeight = parameterDictionary.get("resHeight")
        camName = "Camera" + str(self.camID)

        cv2.namedWindow(camName)
        cap = cv2.VideoCapture(self.camID, cv2.CAP_DSHOW)
        cap.set(cv2.CAP_PROP_FRAME_WIDTH, resWidth)
        cap.set(cv2.CAP_PROP_FRAME_HEIGHT, resHeight)
        cap.set(cv2.CAP_PROP_EXPOSURE, exposure)
        # print('exposure',cap.get(cv2.CAP_PROP_EXPOSURE))
        # if not cap.isOpened():
        #   raise RuntimeError('No camera found at input '+ str(self.camID))
        while True:
            ret1, frame1 = cap.read()
            if ret1 == True:
                if rotNum is not None:
                    frame1 = imutils.rotate_bound(frame1, angle=rotNum)
                cv2.imshow(camName, frame1)
                if cv2.waitKey(1) & 0xFF == 27:
                    # == ord('q') for q
                    break

            else:
                break
        cv2.destroyWindow(camName)


def RunSetup(cam_inputs, rotation_input, paramDict):
    if not cam_inputs:
        raise ValueError("Camera input list (cam_inputs) is empty")

    ulist = []

    for cam_input, cam_rotation in zip(cam_inputs, rotation_input):
        u = VideoSetup(cam_input, paramDict, cam_rotation)
        u.start()
        ulist.append(u)

    for k in ulist:
        k.join()



# -*- coding: utf-8 -*-
"""
Created on Wed Mar  3 12:18:46 2021

@author: Rontc
"""
import tkinter as tk
from tkinter import Tk, Label, Button, Frame, Listbox, Entry
from freemocap.webcam import checkcams
import pickle
import datetime


class recordGUI:
    # this GUI detects how many cameras you have available and presents them in
    # a listbox, and allows you to pick a task for the cameras
    def __init__(self, master):
        self.master = master
        # Detect how many inputs you have available
        available_inputs = checkcams.CreateAvailableCamList()
        number_of_cams = len(available_inputs)
        cam_statement = (
            "You have "
            + str(number_of_cams)
            + " cameras available at inputs "
            + str(available_inputs)
        )

        self.master.title("Choose Your Cameras/Task")
        bottom_frame = Frame(self.master, height=100)
        bottom_frame.pack(side=tk.BOTTOM)
        # ---creates a listbox with all the available camera choices
        self.list_box = Listbox(self.master, selectmode="multiple")
        for i in available_inputs:
            self.list_box.insert(i, str(i))

        # ---setting up the task radiobutton
        # creates the different options and the values associated with them
        options = [("Setup", "setup"), ("Record", "record")]

        # initializes currentTask as a string variable and defaults it to setup
        self.currentTask = tk.StringVar()
        self.currentTask.set("setup")

        # ---display the detection results
        tk.Label(self.master, text=cam_statement, justify=tk.RIGHT).pack()
        tk.Label(self.master, text="Select Your Cameras", justify=tk.RIGHT).pack()
        self.list_box.pack()

        # ---create the task radiobutton
        tk.Label(
            self.master, text="""Choose your task""", justify=tk.LEFT, padx=20
        ).pack()

        for option, val in options:
            tk.Radiobutton(
                self.master, text=option, padx=20, variable=self.currentTask, value=val
            ).pack(anchor=tk.W)

        self.sub_button = Button(bottom_frame, text="Submit", command=self.Submit)
        self.sub_button.pack(side=tk.RIGHT)

    def Submit(self):
        self.taskChoice = self.currentTask.get()
        self.selected = [
            int(self.list_box.get(pos)) for pos in self.list_box.curselection()
        ]
        self.master.destroy()


class SettingsGUI:
    def __init__(
        self,
        master,
        cam_inputs,
        parameter_dictionary,
        rotation_dictionary,
        sessionID_in,
        task,
    ):
        self.master = master
        self.cam_inputs = cam_inputs
        self.parameter_dictionary = parameter_dictionary
        self.rotation_dictionary = rotation_dictionary
        self.sessionID_in = sessionID_in
        self.task = task
        # check if any previous parameters exist
        if self.parameter_dictionary is not None:
            existing_parameters = True
        else:
            existing_parameters = False

        # check to see if there are any stored rotations for the cameras chosen
        rotationValues = self.RotationRetrieval(cam_inputs, rotation_dictionary)

        # create frames to organize rotation inputs, resolution inputs, and parameter inputs
        topFrame = Frame(self.master)
        topFrame.pack(side=tk.TOP)
        rotLabelFrame = Frame(self.master)
        rotLabelFrame.pack()
        resFrame = Frame(self.master)
        resFrame.pack()
        parametersFrame = Frame(self.master)
        parametersFrame.pack()
        bottomFrame = Frame(self.master).pack(side=tk.BOTTOM)

        taskText = "Current Task: " + self.task
        taskLabel = Label(topFrame, text=taskText, font="bold")
        taskLabel.pack()
        # ---Rotation Options. Dynamically creates radio button options depending
        # ---on how many cameras there are
        rotation_options = [
            ("rotate 0", 0),
            ("rotate 90", 90),
            ("rotate 180", 180),
            ("rotate 270", 270),
        ]
        # radVar = tk.IntVar() #Radio Button Variable
        self.rotation_list = []
        rotationRadioButton = {}
        # creates radio buttons for each cam and stores the chosen rotation degree in a list
        for cam, rotDegree in zip(self.cam_inputs, rotationValues):
            current_rotation = tk.IntVar()
            current_rotation.set(rotDegree)
            self.rotation_list.append(current_rotation)
            tk.Label(rotLabelFrame, text="Choose rotation for Cam " + str(cam)).pack()
            for option, degree in rotation_options:
                rotationRadioButton[cam] = tk.Radiobutton(
                    rotLabelFrame, text=option, value=degree, variable=current_rotation
                ).pack()

        # ---Resolution Height and Resolution Width entry
        widthLabel = Label(resFrame, text="Resolution Width").pack(side="left")
        self.resWidthEntry = Entry(resFrame)
        self.resWidthEntry.pack(side="left")
        heightLabel = Label(resFrame, text="Resolution Height").pack(side="left")
        self.resHeightEntry = Entry(resFrame)
        self.resHeightEntry.pack(side="left")

        # ---Exposure,FPS, and codec entry
        exposureLabel = Label(parametersFrame, text="Exposure").pack(side="left")
        self.exposureEntry = Entry(parametersFrame)
        self.exposureEntry.pack(side="left")
        FPSLabel = Label(parametersFrame, text="FPS").pack(side="left")
        self.FPSEntry = Entry(parametersFrame)
        self.FPSEntry.pack(side="left")
        codecLabel = Label(parametersFrame, text="codec").pack(side="left")
        self.codecEntry = Entry(parametersFrame)
        self.codecEntry.pack(side="left")

        # ---SessionID entry- default is the date/time
        if self.task == "record":
            sessionText = "Change SessionID if desired"
            sessionLabel = Label(parametersFrame, text=sessionText).pack()
            self.sessionIDEntry = Entry(parametersFrame)
            self.sessionIDEntry.pack()
            self.sessionIDEntry.insert(0, self.sessionID_in)

        # ---If previous parameters were recieved, insert them into the entry boxes
        if existing_parameters == True:

            self.exposureEntry.insert(0, parameter_dictionary["exposure"])
            self.resWidthEntry.insert(0, parameter_dictionary["resWidth"])
            self.resHeightEntry.insert(0, parameter_dictionary["resHeight"])
            self.codecEntry.insert(0, parameter_dictionary["codec"])
            self.FPSEntry.insert(0, parameter_dictionary["framerate"])

        # ---Submit button to hit when things are finalized
        self.sub_button = Button(bottomFrame, text="Submit", command=self.Submit)
        self.sub_button.pack(side=tk.BOTTOM)

    def Submit(self):
        # get the final entries for all parameters
        # get the rotations for each camera and save them as values of a dictionary
        self.rotation_output = {}
        for count, cam in enumerate(self.cam_inputs):
            self.rotation_output[cam] = self.rotation_list[count].get()

        # built a parameter dictionary by .get()-ing the values from each entry
        self.paramDict = {
            "exposure": int(self.exposureEntry.get()),
            "resWidth": int(self.resWidthEntry.get()),
            "resHeight": int(self.resHeightEntry.get()),
            "framerate": int(self.FPSEntry.get()),
            "codec": str(self.codecEntry.get()),
        }
        # spit out the sessionID
        if self.task == "record":
            self.sessionID_out = self.sessionIDEntry.get()

        self.master.destroy()

    def RotationRetrieval(self, cam_inputs, rotation_dictionary):
        # takes the input cameras and compares them to the dictionary of
        # rotation values from the last session. For any matching cameras,
        # grab the rotation values. Otherwise, the rotation value is None
        rotationValues = []
        for cam in cam_inputs:
            if cam in rotation_dictionary.keys():
                rotationValues.append(rotation_dictionary[cam])
            else:
                rotationValues.append(0)
        return rotationValues


class ProceedToRecordGUI:
    def __init__(self, master, sessionID_in):
        self.master = master
        self.sessionID_in = sessionID_in
        self.continueToRecording = ""
        self.sessionID_out = ""

        sessionText = "Proceed to recording with this sessionID?"
        sessionLabel = Label(master, text=sessionText)
        sessionLabel.pack(side="top")

        self.sessionIDEntry = Entry(master)
        self.sessionIDEntry.pack()
        self.sessionIDEntry.insert(0, self.sessionID_in)

        # master.title("Proceed to Recording?")
        self.proceed_button = Button(text="Proceed", command=self.destroy)
        self.stop_button = Button(text="Quit", command=self.stop)
        self.stop_button.pack(side=tk.RIGHT)
        self.proceed_button.pack(side=tk.RIGHT)

    def stop(self):
        self.master.destroy()

    def destroy(self):
        self.continueToRecording = True
        self.sessionID_out = self.sessionIDEntry.get()
        self.master.destroy()


def RunGUI(sessionID_in, rotation_entry, parameter_entry):

    # ---Get the camera inputs and the task
    root = tk.Tk()
    camera_choice = recordGUI(root)
    root.mainloop()

    cam_inputs = camera_choice.selected
    if not cam_inputs:
        raise ValueError("No camera inputs selected")
    task = camera_choice.taskChoice

    # ---Get all the necessary parameters
    root = tk.Tk()
    recording_settings = SettingsGUI(
        root, cam_inputs, parameter_entry, rotation_entry, sessionID_in, task
    )
    root.mainloop()

    # ---Create the parameters we'll need to run the records
    # rotation_input = list(test.rotation_output.values())

    paramDict = recording_settings.paramDict

    if task == "record":
        sessionID = recording_settings.sessionID_out
    else:
        sessionID = None

    return task, cam_inputs, recording_settings.rotation_output, paramDict, sessionID


def RunProceedtoRecordGUI(sessionID_in):
    root = tk.Tk()
    goToRecording = ProceedToRecordGUI(root, sessionID_in)
    root.mainloop()

    return goToRecording.continueToRecording, goToRecording.sessionID_out
